
// a simple class for ordered lists of strings, implemented as trees
// This version keeps the size of each subtree in each node's count_ attribute.
// count_ is to support an O(log N) implementation of Get(k)

class StrList {  // StrTree with count attribute
     
    // We do not provide a way to set the string in an existing node; 
    // Providing such a method is a bad idea for ordered structures (why?)
    class TreeNode {
    private String item_;
    private int count_;     // should always be the size of this SUBTREE
    private TreeNode left_, right_;
    public TreeNode(String d, TreeNode l, TreeNode r) {
        item_ = d; 
        count_= 1;  // good as long as left_ == right_ == null
        left_ = l; 
        right_= r;
    }
    public String item() {return item_;}
    public int   getcount() {return count_;}
    public TreeNode left()  {return left_;}
    public TreeNode right() {return right_;}
    public void inccount() {count_ += 1;}
    public void setleft  (TreeNode t) {left_ = t;} // doesn't update count_!
    public void setright (TreeNode t) {right_ = t;} // doesn't update count_!
    }

    private TreeNode root_ = null;

    public void inTraverse() {inTraverse(root_, 0);}

    private void inTraverse(TreeNode t, int depth) {
        int indent = 4;
        if (t==null) return;
        inTraverse(t.left(), depth+1);
        String prefix = "";
        for (int i=0; i<depth*indent; i++) prefix += ' ';
        System.out.printf("%s%s (%d)%n", prefix, t.item(), t.getcount());
        inTraverse(t.right(), depth+1);
    }
    
    public int size() {
        return 3;       // Totally wrong, but easy to fix with count_
    }

    // must not be called with val == null
    public void insert(String val) {
        if (root_ == null) {
            root_ = new TreeNode(val, null, null); // sets count_, but not multiplicity_
            return;
        } else {
            rinsert(val, root_);
        }
    }

  // recursive insert: does not get called with t==null
  // returns true if the value was actually inserted (that is, it was NOT already there)
    private boolean rinsert(String str, TreeNode t) {
        int comp = str.compareTo(t.item()); // = 0 if Strings ==, <0 if k<t.key, etc
        if (comp == 0 ) {
            return false;       // already there
        } else if (comp < 0) {      // k < t.item()
            if (t.left() == null) {
                t.setleft(new TreeNode(str, null, null));
                t.inccount();              // count at this node, t, because t.left() grew
                return true;
            } else {
                boolean leftchange = rinsert(str, t.left());      // did left subtree grow?
                if (leftchange) t.inccount();                   // if so, increment count_!!
                return leftchange;
            }
        } else { // val > t.item(); comp > 0
            if (t.right() == null) {        
                t.setright(new TreeNode(str, null, null));   // FIX THIS: call inccount()
                return true;
            } else {                
                boolean rightchange = rinsert(str, t.right());   // FIX THIS!!!
                return false;       // DELETE THIS EVENTUALLY
            }
        }
    }

    public boolean search(String k) {       // same as containsKey()
        return search(k, root_);
    }
    
    private boolean search(String k, TreeNode t) {  // same as containsKey()
        if (t == null) return false;
        int comp = k.compareTo(t.item());
        if (comp == 0) {    // found it
            return true;
        } else if (comp < 0) {  // k < p.item()
            return search(k, t.left());
        } else {
            // finish this part
            return false;   // here only to make this stub compile
        }
    }
    
    public String get(int k) {
        if (k<0 || k >= size()) return null;    
        return get(k, root_);   
    }
    
    // because of the size check above, p should never be null.
    private String get(int k, TreeNode t) {
        int lcount;
        if (t.left() == null) lcount = 0;
        else lcount = t.left().getcount();
        
        return null;  // here only to make this stub compile
    }    
  

}



